﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace _95580
{
    public partial class Form1 : Form
    {

        PageDataTabe paging;
        public class PageDataTabe:INotifyPropertyChanged
        {
            DataTable dt;

            public DataTable DataTable
            {
                get { return dt; }
                
            }

            DataTable origin;
            int currPage = 0;

            public int CurrPage
            {
                get { return currPage; }
                set {
                    if (value > PageCount || value < 0)
                        throw new IndexOutOfRangeException();
                    NotifyPropertyChanged("CurrPage");
                    NotifyPropertyChanged("PageCountFormat");
                    currPage = value;
                    UpdateQuery();
                }
            }

            int rowPerPage = 10;

            public int RowPerPage
            {
                get { return rowPerPage; }
                set {
                    if (value < 1)
                        throw new ArgumentException("Cannot has less thna one row per page");
                    else
                    {
                        rowPerPage = value;
                        NotifyPropertyChanged("RowPerPage");
                        NotifyPropertyChanged("PageCount");
                        NotifyPropertyChanged("PageCountFormat");
                        if (currPage >= PageCount - 1)
                            currPage = PageCount - 1;
                        
                        UpdateQuery();
                    }
                }
            }

            public int PageCount
            {
                get
                {
                    return (int)Math.Ceiling((decimal)origin.Rows.Count / (decimal)rowPerPage);
                }
            }

            public string PageCountFormat
            {
                get
                {
                    return string.Format("{0}/{1}", CurrPage, PageCount - 1);
                }
            }

            public PageDataTabe(ref DataTable dt)
            {
                if (dt.PrimaryKey.Count() < 1)
                    throw new ArgumentException("Data Table has no Primary key");
                this.origin = dt;
                this.currPage = 0;

                UpdateQuery();
            }

            private void UpdateQuery()
            {
                int pr = rowPerPage;
                if (this.origin.Rows.Count - currPage * rowPerPage < rowPerPage)
                    pr = this.origin.Rows.Count - currPage * rowPerPage;

                if (this.dt != null)
                {
                    this.dt.AcceptChanges();
                    this.origin.Merge(this.dt, false, MissingSchemaAction.Ignore);
                }
                this.dt = this.origin.DefaultView.ToTable().Rows.Cast<DataRow>().Take(currPage * rowPerPage + rowPerPage).
                 OrderByDescending(r => r[this.origin.PrimaryKey[0].ColumnName]).Take(pr).OrderBy(r => r[this.origin.PrimaryKey[0].ColumnName]).
                 CopyToDataTable();

                List<DataColumn> pks = new List<DataColumn>();
                foreach (DataColumn col in this.origin.PrimaryKey)
                {
                    if (this.dt.Columns.Contains(col.ColumnName))
                        pks.Add(this.dt.Columns[col.ColumnName]);
                }
                this.dt.PrimaryKey = pks.ToArray();

                NotifyPropertyChanged("DataTable");
                
            }

            #region INotifyPropertyChanged Members

            public event PropertyChangedEventHandler PropertyChanged;

            private void NotifyPropertyChanged(String info)
            {
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs(info));
                }
            }

            #endregion
        }

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            DataTable dt = new DataTable();
            dt.Columns.Add("ID", typeof(int)).AutoIncrement = true;
            dt.PrimaryKey = new DataColumn[] { dt.Columns["ID"] };
            for (int i = 0; i < 5; i++)
            {
                dt.Columns.Add(string.Format("Column{0}", i), typeof(string)).DefaultValue = "Text";
            }

            for (int i = 0; i < 500; i++)
            {
                dt.Rows.Add();
            }

             paging = new PageDataTabe(ref dt);

             ultraGrid1.DataBindings.Add("DataSource", paging, "DataTable");
             label1.DataBindings.Add("Text", paging, "PageCountFormat");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            paging.CurrPage = (paging.CurrPage + 1) % paging.PageCount;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (paging.CurrPage - 1 < 0)
                paging.CurrPage = paging.PageCount-1;
            else
                paging.CurrPage = (paging.CurrPage - 1) ;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            paging.CurrPage = 0;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            paging.CurrPage = paging.PageCount - 1;
        }

        private void ultraNumericEditor1_ValueChanged(object sender, EventArgs e)
        {
            paging.RowPerPage = (int)ultraNumericEditor1.Value;
        }
    }
}
